import shutil
from pathlib import Path
from typing import Annotated

from rich import print
from typer import Argument, Option

from gimad._config import Config
from gimad._utils import CONFIG_NAME, ONEOFF_DIR_NAME, PERMANENT_DIR_NAME


def _create_config() -> None:
    config = Config(db_url="postgresql://user:password@host:port/database")
    config_path = Path(f"{CONFIG_NAME}.json")
    config_path.write_text(config.model_dump_json(indent=2))


def init(
    migration_dir: Annotated[
        str,
        Argument(help="Name of the directory to store migration scripts"),
    ] = "data_migrations",
    force: Annotated[
        bool,
        Option(
            "--force",
            "-f",
            help="Force initialization, can cause data loss if there are files in the related directories",
        ),
    ] = False,
) -> None:
    """Initialize current directory with data migration directory structure"""
    migration_root = Path(migration_dir)
    if force:
        shutil.rmtree(migration_root)
    migration_root.mkdir()
    for d in (PERMANENT_DIR_NAME, ONEOFF_DIR_NAME):
        migration_root.joinpath(d).mkdir()
    _create_config()
    print("[green]Gimad successfully initialized[/green]")



from typing import Annotated

from typer import Option

from gimad._config import load_config
from gimad._db import DatabaseClient
from gimad._runner import Runner


def down(
    count: Annotated[
        int,
        Option("-c", "--count", help="Number of migrations to rollback", min=1),
    ] = 1,
) -> None:
    """Run rollback scripts"""
    config = load_config()
    with DatabaseClient(config.db_url) as client:
        Runner(client).down(count)



import re
from datetime import UTC, datetime
from pathlib import Path
from typing import Annotated

from typer import Option

from gimad._utils import MIGRATION_DIR, ONEOFF_DIR_NAME, PERMANENT_DIR_NAME


def _migration_name(name: str) -> str:
    name = re.sub(r"[^\s\w]", "", name)
    name = re.sub(r"\s+", "_", name)
    return name.lower()


def new(
    name: str,
    permanent: Annotated[
        bool,
        Option(
            help="""
Create a permanent data migration.

By default, the migration generated is one-off. A one-off migration is useful for
existing databases only. For example to remove an abnormal record from the database.

A permanent migration is different, it is useful for new database as well. For example,
adding records to a look-up table is useful to both existing databases as well as new
ones. This is essentially database seeding script.
            """,
        ),
    ] = False,
) -> None:
    """Create new data migration"""

    script_dir = Path(MIGRATION_DIR).joinpath(
        PERMANENT_DIR_NAME if permanent else ONEOFF_DIR_NAME,
    )
    migration_name = _migration_name(name)
    timestamp = datetime.now(tz=UTC).strftime("%Y%m%d_%H%M%S")
    script_name = f"{timestamp}_{migration_name}.py"
    template = Path(__file__).parents[1].joinpath("_template.py")
    script_dir.joinpath(script_name).write_bytes(template.read_bytes())



import os
import sys
from pathlib import Path

from rich import print

PERMANENT_DIR_NAME = "permanent"
ONEOFF_DIR_NAME = "oneoff"

MIGRATION_DIR = Path(os.environ.get("GIMAD_MIGRATION_DIR", "data_migrations")).resolve()
PERMANENT_DIR = MIGRATION_DIR.joinpath(PERMANENT_DIR_NAME)
ONEOFF_DIR = MIGRATION_DIR.joinpath(ONEOFF_DIR_NAME)

CONFIG_NAME = "gimad"


def error(message: str) -> None:
    print(f"[red]ERROR: {message}[/red]", file=sys.stderr)
